from hub_pip.api.RestConnection import RestConnection


class LinkedDataDataService(object):

    rest_connection = None

    def __init__(self, rest_connection):
        self.rest_connection = rest_connection

    def upload_bdio(self, bdio):
        path = "api/bom-import"
        headers = self.rest_connection.headers_jsonld()
        response = self.rest_connection.make_post_request(
            path, bdio, headers=headers)
        response.raise_for_status()
        return response



import json

from hub_pip.api.RestConnection import RestConnection
from hub_pip.api.model.PagedCodeLocationView import PagedCodeLocationView


class CodeLocationsDataService(object):

    rest_connection = None

    def __init__(self, rest_connection):
        self.rest_connection = rest_connection

    def get_paged_code_location_view(self, project_version_view):
        rest_connection = self.rest_connection
        code_location_link = None
        for link in project_version_view.metadata.links:
            if link["rel"] == "codelocations":
                code_location_link = link["href"]
                break
        if code_location_link is None:
            raise Exception(
                "codelocation metadata not found in project version view")

        paged_code_location_view = rest_connection.get_view_from_link(
            PagedCodeLocationView, code_location_link)
        return paged_code_location_view




"""
hub_pip
Usage:
  hub_pip config [<file_path>]
  hub_pip <project-name> <project-version>
        [
        ((-c | --Config) <hub_config.ini>) 
        --Hub-Url=<huburl>
        --Hub-Username=<username>
        --Hub-Password=<password>
        --Hub-Proxy-Host=<None>
        --Hub-Proxy-Port=<None>
        --Hub-Proxy-Username=<None>
        --Hub-Proxy-Password=<None>
        --Hub-Timeout=<120>
        --Hub-ScanTimeout=<300>
        --Hub-CodeLocationName=<None>
        --OutputDirectory=<build/output/>
        --RequirementsFile=<None>
        --IgnoreFailure=<False>
        --CreateFlatDependencyList=<True>
        --CreateTreeDependencyList=<True>
        --CreateHubBdio=<True>
        --DeployHubBdio=<False>
        --CheckPolicies=<False>
        ]
        
Options:
    -h --help                            Show this screen.
    --version                            Show version.
    
Examples:
  hub-pip -c config.ini --DeployHubBdio=True
Help:
  For help using this tool, please open an issue on the Github_pip repository:
  https://github.com/BlackDuckSoftware/hub-python-plugin
"""


"""
hub_pip [
            ((-c | --Config) <hub_config.ini>) 
            --Hub-Url=<huburl>
            --Hub-Username=<username>
            --Hub-Password=<password>
            --Hub-Proxy-Host=<None>
            --Hub-Proxy-Port=<None>
            --Hub-Proxy-Username=<None>
            --Hub-Proxy-Password=<None>
            --Hub-Timeout=<120>
            --Hub-ScanTimeout=<300>
            --Hub-CodeLocationName=<None>
            --OutputDirectory=<build/output/>
            --RequirementsFile=<None>
            --IgnoreFailure=<False>
            --CreateFlatDependencyList=<True>
            --CreateTreeDependencyList=<True>
            --CreateHubBdio=<True>
            --DeployHubBdio=<False>
            --CheckPolicies=<False>
            ]
"""

from inspect import getmembers, isclass
import os

from docopt import docopt

from hub_pip.BlackDuckConfig import BlackDuckConfig
from hub_pip.BlackDuckCore import BlackDuckCore

from . import __version__ as VERSION


def cli():
    """Main CLI entrypoint."""
    options = docopt(__doc__, version=VERSION)

    if options["<project-name>"]:
        options["--Project-Name"] = options["<project-name>"]

    if options["<project-version>"]:
        options["--Project-Version"] = options["<project-version>"]

    if options["config"]:
        if options["<file_path>"]:
            copy_config(path=options["<file_path>"])
        else:
            copy_config()
    else:
        main(options)


def main(options):
    """Build config file from options"""
    config_str = "[Black Duck Config]\n"

    for key, value in options.items():
        if "--" in key and value is not None and value is not "--Config":
            field = key.replace("--", "")
            config_option = field + " = " + str(value) + "\n"
            config_str += config_option

    config = None
    if options["-c"] or options["--Config"]:
        config_file_path = options["<hub_config.ini>"]
        config = BlackDuckConfig.from_file(config_file_path)

    config = BlackDuckConfig.from_string(config_str, black_duck_config=config)

    core = BlackDuckCore(config)
    core.run()


sample_config = """
# This is a sample comment
; This is also a sample comment
# Values are set to None are set to default values

[Black Duck Config]
Hub-Url = None
Hub-Username = None
Hub-Password = None

Hub-Proxy-Host = None
Hub-Proxy-Port = None
Hub-Proxy-Username = None
Hub-Proxy-Password = None

Hub-Timeout = 120
Hub-ScanTimeout = 300

Hub-CodeLocationName = None

OutputDirectory = build/output/
RequirementsFile = None

IgnoreFailure = False
CreateFlatDependencyList = True
CreateTreeDependencyList = True
CreateHubBdio = True
DeployHubBdio = True
CheckPolicies = True

Project-Name = None
Project-Version = None
"""


def copy_config(path=None):
    fullpath = os.path.join(os.getcwd(), "hub_config.ini")
    if path is None:
        path = fullpath
    with open(path, "w+") as file:
        file.write(sample_config)
        print("Created a sample config file @ " + path)



from hub_pip.api.model.MetadataView import MetadataView


class CodeLocationView(object):

    created_at = None
    mapped_project_version = None
    type = None
    updated_at = None
    url = None
    metadata = None

    attribute_map = {
        "created_at": "createdAt",
        "mapped_project_version": "mappedProjectVersion",
        "type": "type",
        "updated_at": "updatedAt",
        "url": "url",
        "metadata": "_meta"
    }

    attribute_schema = {
        "metadata": MetadataView
    }

    def __init__(self):
        pass



class HubServerConfig(object):

    hub_url = None
    hub_username = None
    hub_password = None

    hub_proxy_host = None
    hub_proxy_port = None
    hub_proxy_username = None
    hub_proxy_password = None

    hub_timeout = 120
    hub_scan_timeout = 300

    def __init__(self):
        pass




from hub_pip.BlackDuckPackage import BlackDuckPackage
from hub_pip.BlackDuckSerializer import *
from hub_pip.FileHandler import *
from hub_pip.TreeHandler import TreeHandler
from hub_pip.bdio.BdioComponent import BdioComponent
from hub_pip.bdio.BdioExternalIdentifier import BdioExternalIdentifier
from hub_pip.bdio.BdioProject import BdioProject
from hub_pip.bdio.BdioRelationship import BdioRelationship
from hub_pip.bdio.BillOfMaterials import BillOfMaterials


class Bdio(object):

    tree = None
    code_location_name = None

    def __init__(self, tree, code_location_name):
        self.tree = tree
        self.code_location_name = code_location_name
        if self.code_location_name is None:
            self.code_location_name = tree.name + "/" + \
                tree.version + " Black Duck I/O Export"

    def generate_bdio(self):
        bdio = []
        bom = self._get_bom()
        project = self._get_node(BdioProject(), self.tree)
        components = self._get_components()

        bdio.append(bom)
        bdio.append(project)
        bdio.extend(components)
        return bdio

    def _get_bom(self):
        bom = BillOfMaterials()
        bom.name = self.code_location_name
        bom = map_from_object(bom)
        return bom

    def _get_node(self, node, package):
        node.id = package.get_internal_id()
        node.name = package.name
        node.version = package.version
        node.relationships = self._get_relationships(package)
        node.external_id = map_from_object(package.get_external_id())
        node = map_from_object(node)
        return node

    def _get_components(self):
        components = self.tree.flatten()
        components = [self._get_node(BdioComponent(), pkg)
                      for pkg in components]
        return components

    def _get_relationships(self, package):
        relationships = []
        for pkg in package.dependencies:
            relationship = BdioRelationship()
            relationship.related = pkg.get_internal_id()
            relationship = map_from_object(relationship)
            relationships.append(relationship)
        return relationships

    def write_bdio(self, output_path=None):
        bdio_data = self.generate_bdio()
        bdio_str = json.dumps(bdio_data, indent=4, sort_keys=True)
        if output_path:
            file_name = "bdio"
            if self.tree.name:
                file_name = self.tree.name
            generate_file(
                bdio_str, file_name, output_path, BDIO_EXTENSION)
        return bdio_str



from hub_pip.api.model.MetadataView import MetadataView


class ProjectVersionView(object):

    distribution = None
    license = None
    nickname = None
    phase = None
    release_comments = None
    released_on = None
    source = None
    version_name = None
    metadata = None

    attribute_map = {
        "distribution": "distribution",
        "license": "license",
        "nickname": "nickname",
        "phase": "phase",
        "release_comments": "releaseComments",
        "released_on": "releasedOn",
        "source": "source",
        "version_name": "versionName",
        "metadata": "_meta"
    }

    attribute_schema = {
        "metadata": MetadataView
    }

    def __init__(self):
        pass



class MetadataView(object):

    allow = None
    href = None
    links = None

    attribute_map = {
        "allow": "allow",
        "href": "href",
        "links": "links"
    }

    def __init__(self):
        pass



from hub_pip.bdio.BdioExternalIdentifier import BdioExternalIdentifier


class BlackDuckPackage(object):

    key = None
    name = None
    version = None
    dependencies = []

    def __init__(self, key, name, version, dependencies):
        self.key = key
        self.name = name
        self.version = version
        self.dependencies = dependencies

    def add_dependency(self, package):
        self.dependencies.append(package)

    def render_id(self):
        id_ = self.name + "==" + self.version
        return id_

    def get_internal_id(self):
        return "data:" + self.name + "/" + self.version

    def get_external_id(self):
        external_id = BdioExternalIdentifier()
        external_id.forge = "pypi"
        external_id.external_identifier = self.name + "/" + self.version
        return external_id

    def flatten(self):
        flattened = []
        for pkg in self.dependencies:
            pkg_id = pkg.render_id()
            flattened.append(pkg)
            flattened.extend(pkg.flatten())
        flattened = self._remove_duplicates(flattened)
        return flattened

    def _remove_duplicates(self, packages):
        unique = []
        for package in packages:
            found = False
            for pkg in unique:
                if package.render_id() == pkg.render_id():
                    found = True
                    break
            if not found:
                unique.append(package)
        return unique



import json

from hub_pip.api.RestConnection import RestConnection
from hub_pip.api.model.VersionBomPolicyStatusView import VersionBomPolicyStatusView


class VersionBomPolicyDataService(object):

    rest_connection = None

    def __init__(self, rest_connection):
        self.rest_connection = rest_connection

    def get_version_bom_policy_view(self, project_version_view):
        rest_connection = self.rest_connection
        policy_status_link = None
        for link in project_version_view.metadata.links:
            if link["rel"] == "policy-status":
                policy_status_link = link["href"]
                break
        if policy_status_link is None:
            raise Exception("No metadata found in project version view")

        version_bom_policy_view = rest_connection.get_view_from_link(
            VersionBomPolicyStatusView, policy_status_link)
        return version_bom_policy_view



import uuid


class BillOfMaterials(object):

    id = None
    type = None
    name = None
    external_id = None
    relationships = None
    spec_version = None

    attribute_map = {
        "id": "@id",
        "type": "@type",
        "name": "spdx:name",
        "external_id": "externalIdentifier",
        "relationships": "relationship",
        "spec_version": "spec_version"
    }

    def __init__(self):
        self.id = "uuid:" + str(uuid.uuid4())
        self.type = "BillOfMaterials"
        self.spec_version = "1.1.0"

    def __eq__(self, other):
        if isinstance(other, BillOfMaterials):
            type = self.type == other.type
            name = self.name = other.name
            external_id = self.external_id == other.external_id
            spec_version = self.spec_version == other.spec_version
            relationships = True
            if self.relationships is not None and other.relationships is not None:
                relationships = len(self.relationships) == len(
                    other.relationships)
                if relationships:
                    for relationship in self.relationships:
                        if not relationship in other.relationships:
                            relationships = False
                            break
            elif self.relationships != other.relationships:
                # One is set to None
                relationships = False
            return (type and name and external_id and spec_version and relationships)
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)



class BdioRelationship(object):

    related = None
    relationship_type = "DYNAMIC_LINK"

    attribute_map = {
        "related": "related",
        "relationship_type": "relationshipType",
    }

    def __init__(self):
        pass

    def __eq__(self, other):
        if isinstance(other, BillOfMaterials):
            related = self.related == other.related
            relationship_type = self.relationship_type == other.relationship_type
            return (related and relationship_type)
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)



import os

from setuptools import Command

from docopt import docopt
from hub_pip.BlackDuckConfig import BlackDuckConfig
from hub_pip.BlackDuckCore import BlackDuckCore
from hub_pip.Main import main

from . import __version__ as VERSION


class BlackDuckCommand(Command):

    description = "Setuptools hub_pip"

    user_options = [
        ("Config=", "c", "Path to Black Duck Configuration file"),
        ("Hub-Url=", None, None),
        ("Hub-Username=", None, None),
        ("Hub-Password=", None, None),
        ("Hub-Proxy-Host=", None, None),
        ("Hub-Proxy-Port=", None, None),
        ("Hub-Proxy-Username=", None, None),
        ("Hub-Proxy-Password=", None, None),
        ("Hub-Timeout=", None, None),
        ("Hub-ScanTimeout=", None, None),
        ("Hub-CodeLocationName=", None, None),
        ("OutputDirectory=", None, None),
        ("RequirementsFile=", None, None),
        ("IgnoreFailure=", None, None),
        ("CreateFlatDependencyList=", None, None),
        ("CreateTreeDependencyList=", None, None),
        ("CreateHubBdio=", None, None),
        ("DeployHubBdio=", None, None),
        ("CheckPolicies=", None, None),
        ("Project-Name=", None, None),
        ("Project-Version=", None, None),
    ]

    options = None

    def initialize_options(self):
        """Set default values for options."""
        # Each user option must be listed here with their default value.
        self.Config = None
        self.Hub_Url = None
        self.Hub_Username = None
        self.Hub_Password = None
        self.Hub_Proxy_Host = None
        self.Hub_Proxy_Port = None
        self.Hub_Proxy_Username = None
        self.Hub_Proxy_Password = None
        self.Hub_Timeout = None
        self.Hub_ScanTimeout = None
        self.Hub_CodeLocationName = None
        self.OutputDirectory = None
        self.RequirementsFile = None
        self.IgnoreFailure = None
        self.CreateFlatDependencyList = None
        self.CreateTreeDependencyList = None
        self.CreateHubBdio = None
        self.DeployHubBdio = None
        self.CheckPolicies = None
        self.Project_Name = None
        self.Project_Version = None

    def finalize_options(self):
        if self.Project_Name is None:
            self.Project_Name = self.distribution.get_name()
        if self.Project_Version is None:
            self.Project_Version = self.distribution.get_version()

        self.options = {
            '-c': self.Config,
            '--Config': self.Config,
            '--Hub-Url': self.Hub_Url,
            '--Hub-Username': self.Hub_Username,
            '--Hub-Password': self.Hub_Password,
            '--Hub-Proxy-Host': self.Hub_Proxy_Host,
            '--Hub-Proxy-Port': self.Hub_Proxy_Port,
            '--Hub-Proxy-Username': self.Hub_Proxy_Username,
            '--Hub-Proxy-Password': self.Hub_Proxy_Password,
            '--Hub-Timeout': self.Hub_Timeout,
            '--Hub-ScanTimeout': self.Hub_ScanTimeout,
            '--Hub-CodeLocationName': self.Hub_CodeLocationName,
            '--OutputDirectory': self.OutputDirectory,
            '--RequirementsFile': self.RequirementsFile,
            '--IgnoreFailure': self.IgnoreFailure,
            '--CreateFlatDependencyList': self.CreateFlatDependencyList,
            '--CreateTreeDependencyList': self.CreateTreeDependencyList,
            '--CreateHubBdio': self.CreateHubBdio,
            '--DeployHubBdio': self.DeployHubBdio,
            '--CheckPolicies': self.CheckPolicies,
            '<hub_config.ini>': self.Config,
            '--Project-Name': self.Project_Name,
            '--Project-Version': self.Project_Version,
        }

    def run(self):
        """Run command."""
        main(self.options)


def string_to_boolean(string):
    if string == True:
        return True
    if string == False:
        return False
    if string == "True":
        return True
    elif string == "False":
        return False
    else:
        raise ValueError



from helper import *


class TestConfig:

    def test_valid(self):
        config = get_config(VALID)
        assert(config.hub_server_config.hub_username == "sysadmin")

    def test_invalid(self):
        config = get_config(INVALID)
        assert(config.hub_server_config.hub_username != "sysadmin")

    def test_broken(self):
        broken = False
        try:
            config = get_config(BROKEN)
        except Exception:
            broken = True
        assert(broken)



import json

from hub_pip.api.RestConnection import RestConnection
from hub_pip.api.model.PagedProjectVersionView import \
    PagedProjectVersionView
from hub_pip.api.model.PagedProjectView import PagedProjectView
from hub_pip.api.model.ProjectVersionView import ProjectVersionView
from hub_pip.api.model.ProjectView import ProjectView


class ProjectDataService(object):

    rest_connection = None

    def __init__(self, rest_connection):
        self.rest_connection = rest_connection

    def get_paged_project_view(self, project_name):
        rest_connection = self.rest_connection
        path = "api/projects"
        params = {
            "q": "name:" + project_name
        }
        paged_project_view = rest_connection.get_view_from_path(
            PagedProjectView, path, params=params)
        return paged_project_view

    def get_project_view(self, project_name):
        projects = self.get_paged_project_view(project_name)
        if projects.total_count < 1:
            return None
        return projects.items[0]

    def get_paged_version_view(self, project_view):
        rest_connection = self.rest_connection
        version_link = None
        for link in project_view.metadata.links:
            if link["rel"] == "versions":
                version_link = link["href"]
                break
        if version_link is None:
            raise Exception("No metadata found in project view")
        paged_project_version_view = rest_connection.get_view_from_link(
            PagedProjectVersionView, version_link)
        return paged_project_version_view

    def get_project_version_view(self, project_name, project_version_name):
        project_view = self.get_project_view(project_name)
        if project_view is None:
            return None
        project_version_views = self.get_paged_version_view(project_view)
        project_version_view = None
        if project_version_views.items:
            for version_view in project_version_views.items:
                if version_view.version_name == project_version_name:
                    project_version_view = version_view
                    break
        return project_version_view



from hub_pip.FileHandler import generate_file, FLAT_EXTENSION, TREE_EXTENSION


class TreeHandler(object):
    tree = None

    def __init__(self, tree):
        self.tree = tree

    def render_tree(self,  output_path=None):
        result = self._render_tree_helper(self.tree)
        if output_path:
            generate_file(
                result, self.tree.name, output_path, TREE_EXTENSION)
        return result

    def _render_tree_helper(self, tree_node, layer=1):
        result = tree_node.name + "==" + tree_node.version
        dependencies = sorted(tree_node.dependencies, key=lambda x: x.name)
        for dependency in dependencies:
            result += "\n" + (" " * 4 * layer)
            result += self._render_tree_helper(dependency, layer + 1)
        return result

    def render_flat(self, output_path=None):
        flat_list = self.tree.flatten()
        result = ""
        flat_list.sort(key=lambda x: x.name)
        for node in flat_list:
            result += node.name + "==" + node.version + "\n"

        if output_path:
            generate_file(
                result, self.tree.name, output_path, FLAT_EXTENSION)
        return result



import json
import os

from helper import *
from hub_pip.BlackDuckCore import BlackDuckCore
from hub_pip.BlackDuckSerializer import map_to_object
from hub_pip.FileHandler import get_file_path, BDIO_EXTENSION
from hub_pip.bdio.BdioComponent import BdioComponent
from hub_pip.bdio.BdioProject import BdioProject
from hub_pip.bdio.BillOfMaterials import BillOfMaterials


class TestBdio:

    def test_bdio(self):
        config = get_config(VALID)
        self._test_bdio(config, SMALL_BDIO)

    def _test_bdio_with_requirements(self):
        config = get_config(VALID)
        config.requirements_file_path = REQUIREMENTS
        self._test_bdio(config, BIG_BDIO)

    def _test_bdio(self, config, bdio_sample):
        config.create_hub_bdio = True
        bdio_file_name = get_file_path(
            "hub-pip", config.output_path, BDIO_EXTENSION)

        core = BlackDuckCore(config)
        core.run()

        assert(os.path.exists(bdio_file_name))

        actual_bdio = None
        with open(bdio_file_name) as bdio_file:
            actual_bdio = json.load(bdio_file)

        expected_bdio = None
        with open(bdio_sample) as bdio_file:
            expected_bdio = json.load(bdio_file)

        compare_bom(expected_bdio, actual_bdio)
        compare_project(expected_bdio, actual_bdio)
        compare_components(expected_bdio, actual_bdio)

        cleanup(bdio_file_name)


def get_type(bdio, type, cls):
    node = None
    for component in bdio:
        if component["@type"] == type:
            node = map_to_object(component, cls)
            break
    return node


def get_type_list(bdio, type, cls):
    nodes = []
    for component in bdio:
        if component["@type"] == type:
            node = map_to_object(component, cls)
            nodes.append(node)
    return nodes


def compare_bom(expected_bdio, actual_bdio):
    expected_bom = get_type(expected_bdio, "BillOfMaterials", BillOfMaterials)
    actual_bom = get_type(actual_bdio, "BillOfMaterials", BillOfMaterials)
    expected_bom.name = actual_bom.name
    assert(actual_bom == expected_bom)


def compare_project(expected_bdio, actual_bdio):
    expected_project = get_type(expected_bdio, "Project", BdioProject)
    actual_project = get_type(actual_bdio, "Project", BdioProject)
    expected_project.name = actual_project.name
    expected_project.version = actual_project.version
    expected_project.id = actual_project.id
    expected_project.external_id = actual_project.external_id
    assert(actual_project == expected_project)


def compare_components(expected_bdio, actual_bdio):
    expected_components = get_type_list(
        expected_bdio, "Component", BdioComponent)
    actual_components = get_type_list(actual_bdio, "Component", BdioComponent)
    assert len(actual_components) == len(expected_components)
    for expected_component in expected_components:
        assert(expected_component in actual_components)



from hub_pip.api.model.MetadataView import MetadataView


class ScanSummaryView(object):

    created_at = None
    status = None
    status_message = None
    updated_at = None
    metadata = None

    attribute_map = {
        'created_at': 'createdAt',
        'status': 'status',
        'status_message': 'statusMessage',
        'updated_at': 'updatedAt',
        "metadata": "_meta"
    }

    attribute_schema = {
        "metadata": MetadataView
    }

    def __init__(self):
        pass



import os

import pip

from helper import *
from hub_pip.BlackDuckCore import BlackDuckCore
from hub_pip.FileHandler import get_file_path, BDIO_EXTENSION


class TestDeployment:

    def test_deployment(self):
        config = get_config(VALID)
        # config.requirements_file_path = REQUIREMENTS
        config.create_hub_bdio = True
        config.deploy_hub_bdio = True

        core = BlackDuckCore(config)
        tree = core.run()

        bdio_file_name = get_file_path(
            "hub-pip", config.output_path, BDIO_EXTENSION)
        assert(os.path.exists(bdio_file_name))
        cleanup(bdio_file_name)

        policy_status = core.check_policies(tree)
        assert(policy_status.get_in_violation() == "0")
        assert(policy_status.get_in_violation_but_overridden() == "0")
        assert(policy_status.get_not_in_violation() == "4")



class BdioExternalIdentifier(object):

    forge = None
    external_identifier = None

    attribute_map = {
        "forge": "externalSystemTypeId",
        "external_identifier": "externalId",
    }

    def __init__(self):
        pass



import requests

from hub_pip.BlackDuckSerializer import *
from hub_pip.api.HubServerConfig import HubServerConfig


class RestConnection(object):

    JSPRING = "j_spring_security_check"

    _session = None
    config = None  # HubServerConfig

    def __init__(self, hub_server_config):
        self._session = requests.session()
        if isinstance(hub_server_config, HubServerConfig):
            self.config = hub_server_config

    def get_view_from_path(self, cls, path, params=None, headers=None):
        url = self.build_url(path)
        response_object = self.get_view_from_link(
            cls, url, params=params, headers=headers)
        return response_object

    def get_view_from_link(self, cls, url, params=None, headers=None):
        response = self.make_get_request_link(url, params=params)
        response_json = response.json()
        response_object = self.remap_object(
            response_json, cls)
        return response_object

    def make_get_request(self, path, params=None, headers=None):
        """
        path: str
        queryParameters: dict
        """
        url = self.build_url(path)
        return self.make_get_request_link(url, params=params, headers=headers, proxies=proxies)

    def make_get_request_link(self, url, params=None, headers=None):
        # TODO: check to make sure its a valid url and not a path

        if headers is None:
            headers = self.headers_json()
        proxies = self.get_proxies()
        response = None
        if params:
            response = self._session.get(
                url, params=params, headers=headers, proxies=proxies)
        else:
            response = self._session.get(url, headers=headers, proxies=proxies)
        response.raise_for_status()
        return response

    def make_post_request(self, path, content, headers=None):
        proxies = self.get_proxies()
        url = self.build_url(path)
        response = self._session.post(
            url, content, headers=headers, proxies=proxies)
        return response

    def headers_json(self):
        headers = {"content-type": "application/json"}
        return headers

    def headers_jsonld(self):
        headers = {"content-type": "application/ld+json"}
        return headers

    def get_proxies(self):
        proxies = None
        if self.config.hub_proxy_host is not None:
            protocol = self.config.hub_url.split("://")[0]
            user = self.config.hub_proxy_username
            password = self.config.hub_proxy_password
            host = self.config.hub_proxy_host
            port = self.config.hub_proxy_port
            url = None
            if port:
                address = "{}:{}".format(host, port)
            else:
                address = host
            if user and password:
                url = "{}://{}:{}@{}".format(protocol, user, password, address)
            else:
                url = "{}://{}".format(protocol, address)
            proxies = {
                protocol: url
            }
        return proxies

    def build_url(self, path):
        url = "{}/{}".format(self.config.hub_url, path)
        return url

    def remap_list(self, data, cls):
        data = [map_to_object(item, cls) for item in data]
        return data

    def remap_object(self, data, cls):
        data = map_to_object(data, cls)
        return data
